@@ -4,7 +4,7 @@ use crate::{
         builder::{Builder, Cursor},
         Cell, Grid,
     },
-    undo_redo_buffer, TEXT_LINE_COUNT,
+    undo_redo_buffer, util, TEXT_LINE_COUNT,
 };
 use std::{
     thread,
@@ -28,7 +28,7 @@ fn draw_dark_cell_color(terminal: &mut Terminal, mut cursor_point: Point, grid:
     terminal.reset_colors();
 }
 
-fn get_cell_point_by_cursor_point(cursor_point: Point, builder: &Builder) -> Point {
+fn get_cell_point_from_cursor_point(cursor_point: Point, builder: &Builder) -> Point {
     Point {
         x: (cursor_point.x - builder.cursor.point.x) / 2,
         y: cursor_point.y - builder.cursor.point.y,
@@ -36,13 +36,14 @@ fn get_cell_point_by_cursor_point(cursor_point: Point, builder: &Builder) -> Poi
 }
 
 /// Handles the event and returns a `bool` determing whether execution should be aborted.
-pub fn handle_mouse(
+fn handle_mouse(
     terminal: &mut Terminal,
     event: MouseEvent,
     builder: &mut Builder,
     plot_mode: &mut Option<Cell>,
     editor_toggled: bool,
     starting_time: &mut Option<Instant>,
+    hovered_cell_point: &mut Option<Point>,
 ) -> State {
     match event {
         MouseEvent {
@@ -54,10 +55,12 @@ pub fn handle_mouse(
             point,
         } => {
             if builder.contains(point) {
+                *hovered_cell_point = Some(point);
+
                 let starting_time = starting_time.get_or_insert(Instant::now());
 
-                let cell_point = get_cell_point_by_cursor_point(point, builder);
-                let cell = builder.grid.get_mut_cell(cell_point.x, cell_point.y);
+                let cell_point = get_cell_point_from_cursor_point(point, builder);
+                let cell = builder.grid.get_mut_cell(cell_point);
 
                 if let Some(plot_mode) = *plot_mode {
                     if *cell == plot_mode {
@@ -69,8 +72,6 @@ pub fn handle_mouse(
                         // Overdraw this hovered cell with a dark color
                         draw_dark_cell_color(terminal, point, &builder.grid, cell);
 
-                        terminal.flush();
-
                         return State::Continue;
                     }
                     *cell = plot_mode;
@@ -81,7 +82,7 @@ pub fn handle_mouse(
                         MouseButton::Right => Cell::Crossed,
                     };
                     if *cell == new_plot_mode {
-                        new_plot_mode = Cell::Empty;
+                        new_plot_mode = Cell::default();
                     }
                     *plot_mode = Some(new_plot_mode);
                     *cell = new_plot_mode;
@@ -111,8 +112,6 @@ pub fn handle_mouse(
 
                 // Overdraw this hovered cell with a dark color
                 draw_dark_cell_color(terminal, point, &builder.grid, cell);
-
-                terminal.flush();
             } else {
                 // `plot_mode` won't be reset
             }
@@ -125,8 +124,10 @@ pub fn handle_mouse(
             let _all_clues_solved = builder.draw(terminal);
 
             if builder.contains(point) {
-                let cell_point = get_cell_point_by_cursor_point(point, builder);
-                let cell = builder.grid.get_cell(cell_point.x, cell_point.y);
+                *hovered_cell_point = Some(point);
+
+                let cell_point = get_cell_point_from_cursor_point(point, builder);
+                let cell = builder.grid.get_cell(cell_point);
                 draw_dark_cell_color(terminal, point, &builder.grid, cell);
             }
         }
@@ -138,6 +139,7 @@ pub fn handle_mouse(
     State::Continue
 }
 
+/// Reconstructs the clues associated with the given `cell_point`.
 fn rebuild_clues(terminal: &mut Terminal, builder: &mut Builder, cell_point: Point) {
     builder.clear_clues(terminal);
     builder.grid.horizontal_clues_solutions[cell_point.y as usize] =
@@ -147,7 +149,8 @@ fn rebuild_clues(terminal: &mut Terminal, builder: &mut Builder, cell_point: Poi
 }
 
 /// Handles the event and returns a `State`.
-pub fn handle(
+fn handle(
+    // TODO: this function has too many arguments and should be refactored
     terminal: &mut Terminal,
     event: Event,
     builder: &mut Builder,
@@ -155,6 +158,8 @@ pub fn handle(
     editor: &mut Editor,
     last_notification: Option<&'static str>,
     starting_time: &mut Option<Instant>,
+    hovered_cell_point: &mut Option<Point>,
+    measurement_point: &mut Option<Point>,
 ) -> State {
     match event {
         Event::Mouse(mouse_event) => handle_mouse(
@@ -164,20 +169,127 @@ pub fn handle(
             plot_mode,
             editor.toggled,
             starting_time,
+            hovered_cell_point,
+        ),
+        Event::Key(key_event) => handle_key(
+            terminal,
+            key_event,
+            builder,
+            editor,
+            *hovered_cell_point,
+            measurement_point,
         ),
-        Event::Key(key_event) => handle_key(terminal, key_event, builder, editor),
         Event::Resize => handle_window_resize(terminal, builder, last_notification),
     }
 }
 
 /// This handles all key input for actions like undo, redo, reset and so on.
-pub fn handle_key(
+fn handle_key(
     terminal: &mut Terminal,
     key_event: KeyEvent,
     builder: &mut Builder,
     editor: &mut Editor,
+    hovered_cell_point: Option<Point>,
+    measurement_point: &mut Option<Point>,
 ) -> State {
     match key_event {
+        KeyEvent::Char('a', None) | KeyEvent::Char('A', None) | KeyEvent::Left => {
+            // No grid mutation happened
+            let _all_clues_solved = builder.draw(terminal);
+
+            if let Some(hovered_cell_point) = hovered_cell_point {
+                for x in builder.cursor.point.x..hovered_cell_point.x {
+                    let point = Point {
+                        x,
+                        ..hovered_cell_point
+                    };
+
+                    let cell_point = get_cell_point_from_cursor_point(point, builder);
+                    let cell = builder.grid.get_cell(cell_point);
+
+                    draw_dark_cell_color(terminal, point, &builder.grid, cell);
+                }
+            }
+
+            State::Continue
+        }
+        KeyEvent::Char('d', None) | KeyEvent::Char('D', None) | KeyEvent::Right => {
+            // No grid mutation happened
+            let _all_clues_solved = builder.draw(terminal);
+
+            if let Some(hovered_cell_point) = hovered_cell_point {
+                for x in hovered_cell_point.x..builder.cursor.point.x + builder.grid.size.width * 2
+                {
+                    let point = Point {
+                        x,
+                        ..hovered_cell_point
+                    };
+
+                    let cell_point = get_cell_point_from_cursor_point(point, builder);
+                    let cell = builder.grid.get_cell(cell_point);
+
+                    draw_dark_cell_color(terminal, point, &builder.grid, cell);
+                }
+            }
+
+            State::Continue
+        }
+        KeyEvent::Char('w', None) | KeyEvent::Char('W', None) | KeyEvent::Up => {
+            // No grid mutation happened
+            let _all_clues_solved = builder.draw(terminal);
+
+            if let Some(hovered_cell_point) = hovered_cell_point {
+                for y in builder.cursor.point.y..=hovered_cell_point.y {
+                    let point = Point {
+                        y,
+                        ..hovered_cell_point
+                    };
+
+                    let cell_point = get_cell_point_from_cursor_point(point, builder);
+                    let cell = builder.grid.get_cell(cell_point);
+
+                    draw_dark_cell_color(terminal, point, &builder.grid, cell);
+                }
+            }
+
+            State::Continue
+        }
+        KeyEvent::Char('s', None) | KeyEvent::Char('S', None) | KeyEvent::Down => {
+            // No grid mutation happened
+            let _all_clues_solved = builder.draw(terminal);
+
+            if let Some(hovered_cell_point) = hovered_cell_point {
+                for y in hovered_cell_point.y..builder.cursor.point.y + builder.grid.size.height {
+                    let point = Point {
+                        y,
+                        ..hovered_cell_point
+                    };
+
+                    let cell_point = get_cell_point_from_cursor_point(point, builder);
+                    let cell = builder.grid.get_cell(cell_point);
+
+                    draw_dark_cell_color(terminal, point, &builder.grid, cell);
+                }
+            }
+
+            State::Continue
+        }
+        KeyEvent::Char('q', None) | KeyEvent::Char('Q', None) => {
+            if builder.grid.undo_last_cell() {
+                // It would've already been solved before
+                let _all_clues_solved = builder.draw(terminal);
+            }
+
+            State::Continue
+        }
+        KeyEvent::Char('e', None) | KeyEvent::Char('E', None) => {
+            if builder.grid.redo_last_cell() {
+                // It would've already been solved before
+                let _all_clues_solved = builder.draw(terminal);
+            }
+
+            State::Continue
+        }
         KeyEvent::Char('r', None) | KeyEvent::Char('R', None) => {
             builder.grid.cells.fill_with(Default::default);
             builder
@@ -190,21 +302,51 @@ pub fn handle_key(
 
             State::Continue
         }
-        KeyEvent::Char('q', None) | KeyEvent::Char('Q', None) | KeyEvent::Left(None) => {
-            if builder.grid.undo_last_cell() {
-                // It would've already been solved before
-                let _all_clues_solved = builder.draw(terminal);
-            }
+        KeyEvent::Char('x', None) | KeyEvent::Char('X', None) => {
+            if let Some(hovered_cell_point) = hovered_cell_point {
+                if let Some(some_measurement_point) = *measurement_point {
+                    // The points we have are screen points so now we convert them to values that we can use
+                    // to index the grid.
+                    let start_point =
+                        get_cell_point_from_cursor_point(some_measurement_point, builder);
+                    let end_point = get_cell_point_from_cursor_point(hovered_cell_point, builder);
 
-            State::Continue
-        }
-        KeyEvent::Char('e', None) | KeyEvent::Char('E', None) | KeyEvent::Right(None) => {
-            if builder.grid.redo_last_cell() {
-                // It would've already been solved before
-                let _all_clues_solved = builder.draw(terminal);
-            }
+                    let line_points: Vec<Point> =
+                        util::get_line_points(start_point, end_point).collect();
 
-            State::Continue
+                    set_measured_cells(&mut builder.grid, &line_points);
+
+                    builder
+                        .grid
+                        .undo_redo_buffer
+                        .push(undo_redo_buffer::Operation::Measure(line_points));
+
+                    // Measured cells cannot solve the grid
+                    let _all_clues_solved = builder.draw(terminal);
+
+                    // The cell might not be a measured cell because they are only drawn on
+                    // measured and empty cells
+                    if let Cell::Measured(_) = builder.grid.get_cell(end_point) {
+                        // Overdraw the hovered cell with a dark color
+                        draw_dark_cell_color(
+                            terminal,
+                            hovered_cell_point,
+                            &builder.grid,
+                            Cell::Measured(None),
+                        );
+                    }
+
+                    *measurement_point = None;
+
+                    State::ClearAlert
+                } else {
+                    *measurement_point = Some(hovered_cell_point);
+
+                    State::Alert("Set second measurement point")
+                }
+            } else {
+                State::Continue
+            }
         }
         KeyEvent::Tab => {
             editor.toggle();
@@ -216,9 +358,7 @@ pub fn handle_key(
                 State::Alert("Editor disabled")
             }
         }
-        KeyEvent::Char('s', None) | KeyEvent::Char('S', None) | KeyEvent::Enter
-            if editor.toggled =>
-        {
+        KeyEvent::Enter if editor.toggled => {
             if let Err(err) = editor.save_grid(&builder) {
                 State::Alert(err)
             } else {
@@ -229,12 +369,21 @@ pub fn handle_key(
                 State::Continue
             }
         }
-        KeyEvent::Char(' ', None) => State::Ruler,
         KeyEvent::Esc => State::Exit,
         _ => State::Continue,
     }
 }
 
+pub fn set_measured_cells(grid: &mut Grid, line_points: &[Point]) {
+    for (index, point) in line_points.iter().enumerate() {
+        let cell = grid.get_mut_cell(*point);
+
+        if let Cell::Empty | Cell::Measured(_) = cell {
+            *cell = Cell::Measured(Some(index + 1));
+        }
+    }
+}
+
 fn set_title(terminal: &mut Terminal, title: &str) {
     fn reset_title() {
         thread::spawn(|| {
@@ -268,9 +417,9 @@ pub fn await_fitting_window_size(terminal: &mut Terminal, grid: &Grid) -> State
         (within_width, within_height) => {
             terminal.set_cursor(Point::default());
             if !within_width {
-                terminal.write("Please increase terminal width");
+                terminal.write("Please increase window width or decrease text size (Ctrl and -)");
             } else if !within_height {
-                terminal.write("Please increase terminal height");
+                terminal.write("Please increase window height or decrease text size (Ctrl and -)");
             } else {
                 unreachable!();
             }
@@ -324,7 +473,7 @@ pub fn await_key(terminal: &mut Terminal) {
     }
 }
 
-pub fn await_window_resize(terminal: &mut Terminal) -> State {
+fn await_window_resize(terminal: &mut Terminal) -> State {
     loop {
         let event = terminal.read_event();
         match event {
@@ -338,18 +487,18 @@ pub fn await_window_resize(terminal: &mut Terminal) -> State {
 
 #[must_use]
 pub enum State {
+    /// Execution is to be continued normally.
     Continue,
+    /// The grid has been solved.
     Solved(Duration),
+    /// Display an alert.
     Alert(&'static str),
-    Ruler,
+    /// Clear the alert if present.
+    ClearAlert,
+    /// Exit the program.
     Exit,
 }
 
-struct Ruler {
-    // a good line algorithm lib is needed
-// then maybe you can use that lib to make cell filling smoother too (it can prevent gaps between the filled cells when you move your cursor too quickly)
-}
-
 pub fn r#loop(terminal: &mut Terminal, builder: &mut Builder) -> State {
     let mut plot_mode = None;
     let mut editor = Editor::default();
@@ -359,57 +508,67 @@ pub fn r#loop(terminal: &mut Terminal, builder: &mut Builder) -> State {
 
     let mut starting_time: Option<Instant> = None;
 
-    while let Some(event) = terminal.read_event() {
-        // The order of statements in this loop matters
+    let mut hovered_cell_point: Option<Point> = None;
+    let mut measurement_point: Option<Point> = None;
 
-        if notification_clear_delay != 0 {
-            notification_clear_delay -= 1;
-            if notification_clear_delay == 0 {
-                if let Some(notification_to_clear) = notification {
-                    clear_notification(terminal, builder, notification_to_clear.len());
-                    notification = None;
+    // TODO: refactor above variables into one big struct and/or multiple structs
+
+    loop {
+        if let Some(event) = terminal.read_event() {
+            // The order of statements matters
+
+            if notification_clear_delay != 0 {
+                notification_clear_delay -= 1;
+                if notification_clear_delay == 0 {
+                    if let Some(notification_to_clear) = notification {
+                        clear_notification(terminal, builder, notification_to_clear.len());
+                        notification = None;
+                    }
                 }
             }
-        }
 
-        let state = handle(
-            terminal,
-            event,
-            builder,
-            &mut plot_mode,
-            &mut editor,
-            notification,
-            &mut starting_time,
-        );
+            let state = handle(
+                terminal,
+                event,
+                builder,
+                &mut plot_mode,
+                &mut editor,
+                notification,
+                &mut starting_time,
+                &mut hovered_cell_point,
+                &mut measurement_point,
+            );
 
-        #[cfg(debug_assertions)]
-        {
-            crate::grid::debug::display(terminal, builder);
-        }
+            #[cfg(debug_assertions)]
+            {
+                crate::grid::debug::display(terminal, builder);
+            }
 
-        terminal.flush();
+            terminal.flush();
 
-        match state {
-            State::Continue => continue,
-            State::Alert(new_notification) => {
-                // Draw a new notification. Notifications are cleared after some time.
+            match state {
+                State::Continue => continue,
+                State::Alert(new_notification) => {
+                    // Draw a new notification. Notifications are cleared after some time.
 
-                if let Some(previous_notification) = notification {
-                    clear_notification(terminal, builder, previous_notification.len());
+                    if let Some(previous_notification) = notification {
+                        clear_notification(terminal, builder, previous_notification.len());
+                    }
+                    draw_notification(terminal, builder, new_notification);
+                    notification = Some(new_notification);
+                    notification_clear_delay = 75;
+                    terminal.flush();
                 }
-                draw_notification(terminal, builder, new_notification);
-                notification = Some(new_notification);
-                notification_clear_delay = 75;
-                terminal.flush();
+                State::ClearAlert => {
+                    if let Some(notification_to_clear) = notification {
+                        clear_notification(terminal, builder, notification_to_clear.len());
+                        notification = None;
+                    }
+                }
+                State::Solved(_) | State::Exit => break state,
             }
-            State::Ruler => {
-                // https://old.reddit.com/r/nonograms/comments/nlixfe/any_other_nonogram_apps_especially_for_pcweb_that/
-            }
-            State::Solved(_) | State::Exit => return state,
         }
     }
-
-    unreachable!();
 }
 
 const fn get_notification_y(builder: &Builder) -> u16 {
