@@ -1,5 +1,5 @@
 use itertools::Itertools;
-use terminal::util::{Color, Size};
+use terminal::util::{Color, Point, Size};
 pub mod builder;
 #[cfg(debug_assertions)]
 pub mod debug;
@@ -21,6 +21,10 @@ pub enum Cell {
     Maybed,
     /// Used to mark cells that are certainly empty.
     Crossed,
+    /// Used for indicating cells that were measured using the measurement tool.
+    ///
+    /// When this cell is saved, the index is not preserved.
+    Measured(Option<usize>),
 }
 
 impl Default for Cell {
@@ -30,12 +34,8 @@ impl Default for Cell {
 }
 
 impl From<bool> for Cell {
-    fn from(bool: bool) -> Self {
-        if bool {
-            Cell::Filled
-        } else {
-            Cell::Empty
-        }
+    fn from(filled: bool) -> Self {
+        filled.then(|| Cell::Filled).unwrap_or_default()
     }
 }
 
@@ -46,6 +46,7 @@ impl Cell {
             Cell::Filled => Color::White,
             Cell::Maybed => Color::Blue,
             Cell::Crossed => Color::Red,
+            Cell::Measured(_) => Color::Green,
         }
     }
 
@@ -55,6 +56,7 @@ impl Cell {
             Cell::Filled => Color::Gray,
             Cell::Maybed => Color::DarkBlue,
             Cell::Crossed => Color::DarkRed,
+            Cell::Measured(_) => Color::DarkGreen,
         }
     }
 }
@@ -82,8 +84,8 @@ pub struct Grid {
     pub undo_redo_buffer: UndoRedoBuffer,
 }
 
-fn get_index(width: u16, x: u16, y: u16) -> usize {
-    y as usize * width as usize + x as usize
+fn get_index(width: u16, point: Point) -> usize {
+    point.y as usize * width as usize + point.x as usize
 }
 
 fn get_horizontal_clues(
@@ -92,7 +94,7 @@ fn get_horizontal_clues(
     y: u16,
 ) -> impl Iterator<Item = Clue> + '_ + Clone {
     (0..width)
-        .map(move |x| cells[get_index(width, x, y)] == Cell::Filled)
+        .map(move |x| cells[get_index(width, Point { x, y })] == Cell::Filled)
         .dedup_with_count()
         .filter(|(_, filled)| *filled)
         .map(|(count, _)| count as Clue)
@@ -105,7 +107,7 @@ fn get_vertical_clues(
     x: u16,
 ) -> impl Iterator<Item = Clue> + '_ + Clone {
     (0..height)
-        .map(move |y| cells[get_index(width, x, y)] == Cell::Filled)
+        .map(move |y| cells[get_index(width, Point { x, y })] == Cell::Filled)
         .dedup_with_count()
         .filter(|(_, filled)| *filled)
         .map(|(count, _)| count as Clue)
@@ -160,63 +162,26 @@ impl Grid {
         }
     }
 
-    /// Creates a new grid from the given line of strings.
-    /// Each 1 represents a filled cell.
-    /// Each space represents an empty cell.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// let lines = [
-    ///    "111 1",
-    ///    "11111",
-    ///    "1 111"
-    /// ]
-    /// let grid = Grid::from_lines(lines);
-    /// ```
-    /// It would look like this when solved:
-    /// ```text
-    /// ██████  ██
-    /// ██████████
-    /// ██  ██████
-    /// ```
-    pub fn from_lines(lines: &[&str]) -> Grid {
-        let width = lines.iter().map(|line| line.len()).max().unwrap();
-        let height = lines.len();
-        let size = Size::new(width as u16, height as u16);
-        let mut cells = Vec::<Cell>::with_capacity(size.product() as usize);
-        for line in lines {
-            for char in line.chars() {
-                cells.push(match char {
-                    '1' => Cell::Filled,
-                    ' ' => Cell::Empty,
-                    _ => panic!("the strings must only contain '1' or ' '"),
-                });
-            }
-        }
-        Grid::new(size, cells)
-    }
-
-    fn cell_panic(x: u16, y: u16, index: usize) -> ! {
+    fn cell_panic(point: Point, index: usize) -> ! {
         panic!(
-            "cell access at ({}, {}) with index {} is out of bounds",
-            x, y, index
+            "cell access at {} with index {} is out of bounds",
+            point, index
         );
     }
 
-    pub fn get_cell(&self, x: u16, y: u16) -> Cell {
-        let index = get_index(self.size.width, x, y);
+    pub fn get_cell(&self, point: Point) -> Cell {
+        let index = get_index(self.size.width, point);
         *self
             .cells
             .get(index)
-            .unwrap_or_else(|| Self::cell_panic(x, y, index))
+            .unwrap_or_else(|| Self::cell_panic(point, index))
     }
 
-    pub fn get_mut_cell(&mut self, x: u16, y: u16) -> &mut Cell {
-        let index = get_index(self.size.width, x, y);
+    pub fn get_mut_cell(&mut self, point: Point) -> &mut Cell {
+        let index = get_index(self.size.width, point);
         self.cells
             .get_mut(index)
-            .unwrap_or_else(|| Self::cell_panic(x, y, index))
+            .unwrap_or_else(|| Self::cell_panic(point, index))
     }
 
     pub fn get_horizontal_clues(&self, y: u16) -> impl Iterator<Item = Clue> + '_ + Clone {
@@ -232,6 +197,38 @@ impl Grid {
 mod tests {
     use super::*;
 
+    impl Grid {
+        /// Creates a new grid from the given line of strings.
+        /// A `1` represents a filled cell, a ` ` represents an empty cell.
+        ///
+        /// # Examples
+        ///
+        /// ```
+        /// let lines = [
+        ///    "111 1",
+        ///    "11111",
+        ///    "1 111"
+        /// ]
+        /// let grid = Grid::from_lines(lines);
+        /// ```
+        fn from_lines(lines: &[&str]) -> Grid {
+            let width = lines.iter().map(|line| line.len()).max().unwrap();
+            let height = lines.len();
+            let size = Size::new(width as u16, height as u16);
+            let mut cells = Vec::<Cell>::with_capacity(size.product() as usize);
+            for line in lines {
+                for char in line.chars() {
+                    cells.push(match char {
+                        '1' => Cell::Filled,
+                        ' ' => Cell::Empty,
+                        _ => panic!("the strings must only contain '1' or ' '"),
+                    });
+                }
+            }
+            Grid::new(size, cells)
+        }
+    }
+
     #[test]
     fn test_squared_grid() {
         let grid = Grid::from_lines(&[
@@ -240,11 +237,6 @@ mod tests {
             "1111 11  1",
             "1 11 1  11",
             "1  111  11",
-            "          ",
-            "          ",
-            "          ",
-            "          ",
-            "          ",
         ]);
 
         assert_eq!(
@@ -255,11 +247,6 @@ mod tests {
                 vec![4, 2, 1],
                 vec![1, 2, 1, 2],
                 vec![1, 3, 2],
-                vec![],
-                vec![],
-                vec![],
-                vec![],
-                vec![]
             ]
         );
 
@@ -290,10 +277,6 @@ mod tests {
                 "1 1 ",
                 "1  1",
                 "  1 ",
-                "    ",
-                "    ",
-                "    ",
-                "    ",
             ]);
 
         assert_eq!(
@@ -305,10 +288,6 @@ mod tests {
                 vec![1, 1],
                 vec![1, 1],
                 vec![1],
-                vec![],
-                vec![],
-                vec![],
-                vec![]
             ]
         );
 
